<?php

namespace InterludeSante\Rating;

use ACFComposer\ACFComposer;

class Rating
{

    /**
     * The single instance of the class.
     *
     * @since  1.0
     * @access protected
     */
    protected static $instance = null;

    /**
     * Taxonomy name.
     *
     * @since  1.0
     * @access private
     */
    private $taxoName;


    /**
     * Post Type name.
     *
     * @since  1.0
     * @access private
     */
    private $postName;

    /**
     * Main instance.
     *
     * Ensures only one instance is loaded or can be loaded.
     *
     * @since  1.0
     * @access public
     * @param  string $path
     *
     * @return Main instance.
     */
    public static function instance()
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function __construct()
    {
        $this->postName = "rating";
        $this->taxoName = "ratingType";
        $this->registerHooks();
    }

    /**
     * Retrieve the name of the plugin.
     *
     * @since  1.0
     * @access public
     *
     * @return string
     */
    public function getName()
    {
        $plugin_data = get_file_data($this->path, ['Plugin Name' => 'Plugin Name'], 'plugin');
        return $plugin_data['Plugin Name'];
    }

    /**
     * Registering WordPress Hooks.
     *
     * @since  1.0
     * @access public
     */
    public function registerHooks()
    {
        add_action('init', [$this, 'registerPostType']);
        add_action('init', [$this, 'registerTaxo']);
        add_action('init', [$this, 'registerAcfFields']);
        add_action('init', [$this, 'registerTerms']);

        add_action('admin_menu', [$this, 'addSubMenuPage']);

        $this->manageColumnService();
        $this->manageColumnRate();
        $this->manageColumnOrder();

        add_action('views_edit-rating', [$this, 'addRatingAverage'], 1000);
    }

    /**
     * Create sub menu page of the plugin.
     *
     * @since  1.0
     * @access public
     */
    public function addSubMenuPage()
    {
        $terms = get_terms(array(
            'taxonomy' => $this->taxoName,
            'hide_empty' => false,
        ));

        if (!empty($terms)) {
            foreach ($terms as $term) {
                $menu_slug = 'edit.php?post_type='.$this->postName.'&'.strtolower($this->taxoName).'='.$term->slug;
                add_submenu_page(
                    'edit.php?post_type='.$this->postName,
                    $term->name,
                    $term->name,
                    'delete_others_pages',
                    $menu_slug
                );
            }
        }
    }

    /**
     * Manage column ordering.
     *
     * @since  1.0
     * @access public
     */
    public function manageColumnOrder()
    {
        add_filter('manage_rating_posts_columns', function ($columns) {
            $date = $columns['date'];
            unset($columns['date']);
            $columns['date'] = $date;
            return $columns;
        });
    }

    /**
     * Add the service column.
     *
     * @since  1.0
     * @access public
     */
    public function manageColumnService()
    {
        add_filter('manage_rating_posts_columns', function ($columns) {
            return array_merge($columns, ['service' => __('Service', 'textdomain')]);
        });

        add_action('manage_rating_posts_custom_column', function ($column_key, $post_id) {
            if ($column_key == 'service') {
                $service = get_post_meta($post_id, 'service', true);
                $service = $service ? $service : ' ';
                echo (!empty($service)) ? sprintf(__('%s', 'textdomain'), $service) : __('Unknown', 'textdomain');
            }
        }, 10, 2);
    }

    /**
     * Add the rate column and make it sortable
     *
     * @since  1.0
     * @access public
     */
    public function manageColumnRate()
    {
        add_filter('manage_rating_posts_columns', function ($columns) {
            return array_merge($columns, ['rate' => __('Note', 'textdomain')]);
        });

        add_action('manage_rating_posts_custom_column', function ($column_key, $post_id) {
            if ($column_key == 'rate') {
                $rate = get_post_meta($post_id, 'rate', true);
                echo (!empty($rate)) ? sprintf(__('%s', 'textdomain'), $rate) : __('Unknown', 'textdomain');
            }
        }, 10, 2);

        add_filter('manage_edit-rating_sortable_columns', function ($columns) {
            $columns['rate'] = 'rate';
            return $columns;
        });

        add_action('pre_get_posts', function ($query) {
            if (!is_admin()) {
                return;
            }

            $orderby = $query->get('orderby');
            if ($orderby == 'rate') {
                $query->set('meta_key', 'rate');
                $query->set('orderby', 'meta_value_num');
            }
        });
    }


    /**
     * Registering ACF field to rating post type.
     *
     * @since  1.0
     * @access public
     */
    public function registerAcfFields()
    {
        ACFComposer::registerFieldGroup([
            'name' => 'rate',
            'title' => 'rate',
            'style' => 'seamless',
            'fields' =>
            [
                [
                    'label' => 'Note',
                    'name' => 'rate',
                    'type' => 'number',
                    'min' => 1,
                    'max' => 5,
                    'step' => 1
                ],
                [
                    'label' => 'Service',
                    'name' => 'service',
                    'type' => 'text',
                ],
                [
                    'label' => 'Commentaire',
                    'name' => 'comment',
                    'type' => 'textarea',
                    'rows' => 8,
                    'new_lines' => 'wpautop'
                ],
            ],
            'location' => [
                [
                    [
                        'param' => 'post_type',
                        'operator' => '==',
                        'value' => $this->postName,
                    ],
                ],
            ],
        ]);
    }



    /**
     * Registering ratingType taxonomy to rating post type.
     *
     * @since  1.0
     * @access public
     */
    public function registerTaxo()
    {
        $labels = [
            'name'                       => 'Types de note',
            'singular_name'              => 'Type de note',
            'menu_name'                  => 'Types de note',
            'search_items'               => 'Rechercher des types de note',
            'popular_items'              => 'Types de note populaires',
            'all_items'                  => 'Tous les types de note',
            'parent_item'                => 'Type de note parent',
            'parent_item_colon'          => 'Type de note parent :',
            'edit_item'                  => 'Modifier le type de note',
            'view_item'                  => 'Voir le type de note',
            'update_item'                => 'Mettre à jour le type de note',
            'add_new_item'               => 'Ajouter un nouveau type de note',
            'new_item_name'              => 'Nom du nouveau type de note',
            'separate_items_with_commas' => 'Séparer les types de note avec des virgules',
            'add_or_remove_items'        => 'Ajouter ou supprimer des types de note',
            'choose_from_most_used'      => 'Choisir parmi les types de note les plus utilisés',
            'not_found'                  => 'Aucun type de note trouvé',
            'no_terms'                   => 'Aucun type de note',
            'filter_by_item'             => 'Filtrer par type de note',
            'items_list_navigation'      => 'Navigation de la liste des types de note',
            'items_list'                 => 'Liste des types de note',
            'most_used'                  => 'Plus utilisés',
            'back_to_items'              => '&larr; Aller aux types de note',
            'item_link'                  => 'Lien de type de note',
            'item_link_description'      => 'Un lien vers un type de note',
        ];
        $args = [
            'labels'                     => $labels,
            'hierarchical'               => true,
            'public'                     => true,
            'publicly_queryable'         => true,
            'show_in_menu'               => true,
            'show_ui'                    => true,
            'show_admin_column'          => true,
            'show_in_nav_menus'          => true,
            'show_tagcloud'              => true,
            'show_in_rest'               => true,

        ];

        register_taxonomy($this->taxoName, [$this->postName], $args);
    }

    /**
     * Registering rating post type.
     *
     * @since  1.0
     * @access public
     */
    public function registerPostType()
    {
        $labels = [
            'name'                     => 'Notes',
            'singular_name'            => 'Note',
            'menu_name'                => 'Notes',
            'add_new'                  => 'Ajouter',
            'add_new_item'             => 'Ajouter une nouvelle note',
            'edit_item'                => 'Modifier la note',
            'new_item'                 => 'Nouvelle note',
            'view_item'                => 'Voir la note',
            'view_items'               => 'Voir les notes',
            'search_items'             => 'Rechercher dans les notes',
            'not_found'                => 'Aucune note trouvée.',
            'not_found_in_trash'       => 'Aucune note trouvée dans la corbeille',
            'parent_item_colon'        => 'Note parente :',
            'all_items'                => 'Toutes les notes',
            'archives'                 => 'Archive des notes',
            'attributes'               => 'Attributs de la note',
            'insert_into_item'         => 'Insérer dans la note',
            'uploaded_to_this_item'    => 'Téléversé sur cette note',
            'feature_image'            => 'Image à la une',
            'set_featured_image'       => 'Utiliser comme image à la une',
            'remove_featured_image'    => 'Supprimer l’image à la une',
            'use_featured_image'       => 'Utiliser comme image à la une',
            'filter_items_list'        => 'Filtrer la liste des notes',
            'filter_by_date'           => 'Filtrer par date',
            'items_list_navigation'    => 'Navigation de la liste des notes',
            'items_list'               => 'Liste des notes',
            'item_published'           => 'Note publiée',
            'item_published_privately' => 'Note mise en ligne en privé.',
            'item_reverted_to_draft'   => 'Note reconvertie en brouillon.',
            'item_scheduled'           => 'Note planifiée.',
            'item_updated'             => 'Note mise à jour.',
            'item_link'                => 'Lien de note',
            'item_link_description'    => 'Un lien vers une note',
        ];
        $args = [
            'label'                 => __('Post Type', 'flynt'),
            'description'           => __('Post Type Description', 'flynt'),
            'labels'                => $labels,
            'supports'              => ['title', 'revisions'],
            'taxonomies'            => ['ratingType'],
            'hierarchical'          => false,
            'public'                => true,
            'show_ui'               => true,
            'show_in_menu'          => true,
            'menu_position'         => 50,
            'show_in_admin_bar'     => true,
            'show_in_nav_menus'     => true,
            'can_export'            => true,
            'has_archive'           => true,
            'exclude_from_search'   => false,
            'publicly_queryable'    => true,
            'capability_type'       => 'page',
            'show_in_rest'          => true,
            'menu_icon'             => 'dashicons-star-half',
        ];

        register_post_type($this->postName, $args);
    }


    /**
     * Registering ratingType terms.
     *
     * @since  1.0
     * @access public
     */
    public function registerTerms()
    {
        wp_insert_term('App', $this->taxoName);
        wp_insert_term('Stage', $this->taxoName);
    }


    /**
     * Add average to rating page.
     *
     * @since  1.0
     * @access public
     */
    public function addRatingAverage()
    {
        $term = $_GET[strtolower($this->taxoName)];
        if (!empty($term)) {
            $numberOfRate = 0;
            $totalOfRate = 0;
            $rate = 0;

            $the_query = new \WP_Query(array(
                'post_type' => $this->postName,
                'tax_query' => array(
                    array(
                        'taxonomy' => $this->taxoName,
                        'field' => 'slug',
                        'terms' => $term,
                    )
                ),
            ));

            if ($the_query->have_posts()) {
                while ($the_query->have_posts()) {
                    $the_query->the_post();
                    $rating_id = get_the_ID();
                    $rate = get_field("rate", $rating_id);
                    $numberOfRate ++;
                    $totalOfRate += $rate;
                }
                $rate = $totalOfRate / $numberOfRate;
                $rate = round($rate, 2, PHP_ROUND_HALF_UP);
            }


            $htmlAverage = '<div class="rating-container">
            <span class="row-title">Moyenne : </span>
            <span class="row-title rate"> '.$rate.'/5</>
            </div>';

            echo '<script>
                    $("'.$htmlAverage.'" ).prependTo( ".search-box"");
                 </script>
                ';
            echo '<style>
                .rating-container {
                    width: 130px;
                    height: 32px;
                    background-color: #ffffff;
                    border: solid 1px #c3c4c7;
                    border-radius: 4px;
                    display: flex;
                    justify-content: flex-start;
                    align-items: center;
                    padding: 0 8px;
                }
                .average-rate {
                    margin-left: 10px;
                    font-weight: bold;
                }
                </style>';
            echo '
                    <div class="rating-container">
                        <span class="row-title">Moyenne : </span>
                        <span class="row-title average-rate"> '.$rate.'/5</>
                    </div>';
        }
    }
}
